import React, { ReactElement, useMemo } from 'react';
import styles from './buttons.module.scss';
import Icon, { Sizes } from '../icon';
import { forwardRef } from 'react';

type ButtonProps = {
  as?: 'button' | 'a' | 'div';
  variantType?: 'primary' | 'secondary' | 'danger';
  variant?: 'solid' | 'outline' | 'ghost';
  size?: '2xs' | 'xs' | 'sm' | 'md' | 'lg';
  label?: string;
  rounded?: boolean;
  disabled?: boolean;
  block?: boolean;
  onClick?: (e: any) => void;
  href?: string;
  iconComponent?: React.ReactElement;
  iconPosition?: 'leading' | 'trailing';
  ariaLabel?: string;
  children?: React.ReactNode;
  className?: string;
};

export const Button = React.forwardRef<HTMLButtonElement | HTMLAnchorElement | HTMLDivElement, ButtonProps>(
  (
    {
      as = 'button',
      variantType = 'secondary',
      variant = 'solid',
      label,
      size = 'md',
      rounded = false,
      disabled = false,
      onClick,
      href,
      block = false,
      iconComponent,
      iconPosition = 'leading',
      ariaLabel,
      className,
      children,
      ...props
    },
    forwardedRef,
  ) => {
    let typeClass = useMemo(() => {
      switch (variantType) {
        case 'primary':
          return styles['btn-primary'];
        case 'secondary':
          return styles['btn-secondary'];
        case 'danger':
          return styles['btn-danger'];
        default:
          return styles['btn-secondary'];
      }
    }, [variantType]);

    let variantClass = useMemo(() => {
      switch (variant) {
        case 'solid':
          return styles['btn-solid'];
        case 'outline':
          return styles['btn-outline'];
        case 'ghost':
          return styles['btn-ghost'];
        default:
          return styles['btn-solid'];
      }
    }, [variant]);

    let sizeClass = useMemo(() => {
      switch (size) {
        case '2xs':
          return styles['btn-2xs'];
        case 'xs':
          return styles['btn-xs'];
        case 'sm':
          return styles['btn-sm'];
        case 'md':
          return styles['btn-md'];
        case 'lg':
          return styles['btn-lg'];
        default:
          return styles['btn-md'];
      }
    }, [size]);

    const iconSize = useMemo(() => {
      switch (size) {
        case '2xs':
          return 12;
        case 'xs':
          return 16;
        case 'sm':
          return 20;
        case 'md':
          return 24;
        case 'lg':
          return 28;
        default:
          return 24;
      }
    }, [size]);

    const blockClass = useMemo(() => (block ? styles['btn-block'] : ''), [block, styles]);
    const roundedClass = useMemo(() => (rounded ? styles['btn-rounded'] : ''), [rounded, styles]);

    const icon = useMemo(
      () => (iconComponent ? <Icon component={iconComponent} size={iconSize} className="ml-auto" /> : null),
      [iconComponent, iconSize],
    );

    const iconOnlyClass = useMemo(
      () => (iconComponent && !label && !children ? styles['btn-icon-only'] : ''),
      [iconComponent, label, children],
    );

    const ButtonComponent = as;

    const isAnchor = as === 'a';

    return (
      <ButtonComponent
        className={`${styles.btn} ${typeClass} ${variantClass} ${sizeClass} ${blockClass} ${roundedClass} ${iconOnlyClass} ${className}`}
        onClick={onClick}
        disabled={disabled}
        aria-label={ariaLabel}
        href={isAnchor ? href : undefined}
        ref={forwardedRef as React.RefObject<any>}
        {...props}
      >
        {iconPosition === 'leading' && icon}
        {label && <span>{label}</span>}
        {children && <span>{children}</span>}
        {iconPosition === 'trailing' && icon}
      </ButtonComponent>
    );
  },
);

type ButtonGroupProps = {
  children: React.ReactNode;
};

export function ButtonGroup({ children }: ButtonGroupProps) {
  return <div>{children}</div>;
}

type ButtonItemProps = {
  icon: React.ReactNode;
  onClick: () => void;
};

export function ButtonGroupItem({ icon, onClick }: ButtonItemProps) {
  return (
    <div onClick={onClick} className="rounded-sm bg-secondary-50 p-2">
      <span>{icon}</span>
    </div>
  );
}
